
param([string]$computerToCheck=("localhost"),
       [string]$domain="",
       [string]$user="",
       [string]$password=""
        
      )
$checkResults = New-Object System.Collections.Generic.List[System.Object]

$OldEAP = $ErrorActionPreference
$ErrorActionPreference = 'silentlycontinue';


#Block of Script to check for .Net Requirements
$DotNetRequirements = {
    $NetRegKey = Get-Childitem 'HKLM:\SOFTWARE\Microsoft\NET Framework Setup\NDP\v4\Full'
	#Write-Host $NetRegKey
    $Release = $NetRegKey.GetValue("Release")
    Switch ($Release) {
   378389 {$NetFrameworkVersion = ".Net Version 4.5 "}
   378675 {$NetFrameworkVersion = ".Net Version 4.5.1 "}
   378758 {$NetFrameworkVersion = ".Net Version 4.5.1 "}
   379893 {$NetFrameworkVersion = ".Net Version 4.5.2 "}
   393295 {$NetFrameworkVersion = ".Net Version 4.6 "}
   393297 {$NetFrameworkVersion = ".Net Version 4.6 "}
   394254 {$NetFrameworkVersion = ".Net Version 4.6.1 "}
   394271 {$NetFrameworkVersion = ".Net Version 4.6.1 "}
   394802 {$NetFrameworkVersion = ".Net Version 4.6.2 "}
   394806 {$NetFrameworkVersion = ".Net Version 4.6.2 "}
   460798 {$NetFrameworkVersion = ".Net Version 4.7 "}
   460805 {$NetFrameworkVersion = ".Net Version 4.7 "}
   461308 {$NetFrameworkVersion = ".Net Version 4.7.02556 "}
   461808 {$NetFrameworkVersion = ".Net Version 4.7.2 "}
   461814 {$NetFrameworkVersion = ".Net Version 4.7.2 "}
   528040 {$NetFrameworkVersion = ".Net Version 4.8 "}
   528049 {$NetFrameworkVersion = ".Net Version 4.8 "}
   Default {$NetFrameworkVersion = "Net Framework 4.5 or later is not installed. Fails Requirement"}
    }
			
    $Object = [PSCustomObject]@{
	
        NETFrameworkVersion = $NetFrameworkVersion
    }
    $Object
}

# Print User message using String Array $message
function DisplayErrorToUser {
    param(
        [Parameter( `
            Mandatory=$True, `
            Valuefrompipeline = $true)]
        [String]$message     
       
       
    )
    begin {
        $window_private_data = (Get-Host).PrivateData;      
        $saved_background_color = $window_private_data.VerboseBackgroundColor
        $saved_foreground_color = $window_private_data.VerboseForegroundColor
        # setting the new colors
        
        $window_private_data.VerboseForegroundColor = "Red";
    }
    process {
        #foreach ($Message in $Message) {          
            Write-Verbose -Message $message -Verbose;           
        #}
    }
    end {
      $window_private_data.VerboseBackgroundColor = $saved_background_color;
      $window_private_data.VerboseForegroundColor = $saved_foreground_color;
    }

} # end PrintMessageToUser
function DisplayResultToUser {
    param(
        [Parameter( `
            Mandatory=$True, `
            Valuefrompipeline = $true)]
        [String]$message     
       
       
    )
    begin {
        $window_private_data = (Get-Host).PrivateData;      
        $saved_background_color = $window_private_data.VerboseBackgroundColor
        $saved_foreground_color = $window_private_data.VerboseForegroundColor
        # setting the new colors
        
        $window_private_data.VerboseForegroundColor = "Green";
    }
    process {
        #foreach ($Message in $Message) {          
            Write-Verbose -Message $message -Verbose;           
        #}
    }
    end {
      $window_private_data.VerboseBackgroundColor = $saved_background_color;
      $window_private_data.VerboseForegroundColor = $saved_foreground_color;
    }

} 



try
{
$VerbosePreference = "Continue"
DisplayResultToUser("Checking the .Net Framework Requirement" );
Write-Progress -Activity "Checking .Net Framework Requirement" -PercentComplete 16
$dotNetRes =  .$DotNetRequirements;
$res = $dotNetRes.NETFrameworkVersion;

 if ($res -match "Fails") {
   
        
        $checkResults.Add("Does Meet Minimum .Net Requimement (4.5 Version is Required)");
        
   }
   else{       
          DisplayResultToUser("Result: Meets Minimum .Net Requirement - $res Found"  ) ;
   }


DisplayResultToUser( "Checking version of PowerShell");
Write-Progress -Activity "Checking Powershell Requirement" -PercentComplete 32

$psVersion =  $PSVersionTable.psversion
$psVersionNumber = "$psVersion.Major . $psVersion.Minor" 

if ($psVersionNumber -ge 3.0 ) {
    DisplayResultToUser("Result: Meets Minimum Powershell Version - $psVersion Found");
}
else{    

    $checkResults.Add( "Does Meet Minimum Powershell Version (3.0 Version is Required) Current Version is $psVersion");
}


DisplayResultToUser("Checking Execution Policy");
Write-Progress -Activity "Checking  Execution Policy Requirement" -PercentComplete 48

$ExecutionPolicy = get-executionpolicy
$allowablePolicies = "RemoteSigned","Unrestricted","Bypass";

if ( $allowablePolicies  -notcontains $ExecutionPolicy){ 
 
 $checkResults.Add( "Current Policy ( $ExecutionPolicy )  Will not allow remote scripts to be executed");
 exit;
}
else{
    DisplayResultToUser( "Result: Execution Policy is set to $ExecutionPolicy");
}


$domainName = ""
$userName =  ""
$pw =  ""

if ($domain.Length -gt 1 -and $user.Length -gt 1){
    $pass= $password|ConvertTo-SecureString -AsPlainText -Force
    $domainUser = "$user@$domain";
    $reqCredential = New-Object   System.Management.Automation.PsCredential($domainUser,$pass)
    $domainName = $reqCredential.username.Split("@")[1] 
    $userName = $reqCredential.username.Split("@")[0] 
    $pw = $reqCredential.GetNetworkCredential().password
}
else{
    $reqCredential = Get-Credential -Message "Enter Domain Account for Testing Requirements"
    if ($reqCredential){
        
        $domainName = $reqCredential.username.Split("{\}")[0] 
        $userName = $reqCredential.username.Split("{\}")[1] 
        $pw = $reqCredential.GetNetworkCredential().password
    }
    else{
    
        $checkResults.Add("No Credentials have been provided for Testing Domain Authentication");
    }
    
} 

if ($userName.Length > 0){

    DisplayResultToUser("Checking User $userName Is in the Correct AD Group");
    Write-Progress -Activity "Checking  User Requirement" -PercentComplete 65


    $members = Get-ADGroupMember -Identity "Domain Admins" -Recursive | Select -ExpandProperty SamAccountName

    If ($members -contains $userName) {
         DisplayResultToUser("Result: $userName is a member of Domain Administration Group");
    } Else {
        #Write-Warning  "     Result: $userName is not a member of  Domain Administration Group";
        $checkResults.Add("$userName is not a member of  Domain Administration Group");
    }

}

    if ($computerToCheck -ne "localhost"){
        
        DisplayResultToUser("Checking to See if $computerToCheck can be resolved by Name");
        Write-Progress -Activity "Checking  Remote Session" -PercentComplete 75
        $resolveComputer = [bool] (Resolve-DnsName $computerToCheck);
        if ( $resolveComputer) {
            DisplayResultToUser("Result: $computerToCheck  can be resolved by name");
        }
        else{

         $checkResults.Add("$computerToCheck Could not be resolved by name");    
        }


        DisplayResultToUser("Checking for Access to Perform Remote Session Commands");
        Write-Progress -Activity "Checking  Remote Session" -PercentComplete 90
        
        $testSession = New-PSSession -Computer $computerToCheck -Credential $reqCredential
        if(-not($testSession)){
        
            $checkResults.Add("Unable to perform Remote Session Commands on $computerToCheck");
        }
        else{
            DisplayResultToUser("Result: $computerToCheck is accessible!")    
        }        


        DisplayResultToUser("Checking Connectivity to Computer - $computerToCheck");
        Write-Progress -Activity "Checking Connectivity to Computer - $computerToCheck" -PercentComplete 100
       
        if ( (Test-Connection  $computerToCheck -Quiet -Count 2) -eq "True")  {
            DisplayResultToUser("Result: $computerToCheck  could be pinged");
        }
        else{

         $checkResults.Add("$computerToCheck Is Unreachable at this time");    
        }
    }
    

}
catch {
    Write-Warning  $_.Exception.Message;
    Write-Warning  $_.Exception.ItemName;
}
Finally{
Remove-PSSession $testSession
$ErrorActionPreference = $OldEAP
 if ($checkResults.Count -gt  0)
 {
    DisplayResultToUser("   ");
    DisplayResultToUser("   ");
    DisplayErrorToUser("*****************************************************");
    DisplayErrorToUser("Please fix the following issues and re-run the script");
    foreach($warning in $checkResults){
       DisplayErrorToUser( $warning)

    }
    DisplayErrorToUser("*****************************************************");
 }
 else{
     DisplayResultToUser("All the Minimum Requirements Have Been Met");
 
 }
}





